#!/bin/sh

# This file configuring and start Wireless client mode and connect to AP

# Command line arguments:
#   $1 interface name
#   $2 wireless ssid
#   $3 password

#############################
#########  Params  ##########
#############################

INTERFACE="$1"
SSID="$2"
PASSWORD="$3"
DRIVER="nl80211"
WPA_CONF="wpa_client.conf"
WLAN_HOSTNAME="wlan_hostname"

#############################
######### Functions #########
#############################

ip link set dev $INTERFACE down

create_config_file_wifi()
{
    FILE=$1
    WIFI_SSID=$2
    WIFI_PASS=$3

    #Delete file
    if [ -e "$FILE" ] ; then
        rm "$FILE"
    fi

    echo "# This file has been generated by HomeKit engine. DO NOT EDIT!
    ctrl_interface=/var/run/wpa_supplicant
    update_config=1
    ap_scan=1
    eapol_version=1
    network={
    scan_ssid=1
    mode=0
    priority=20
    ssid=\"$WIFI_SSID\"
    " >> "$FILE"
    
    #Append password if in args
    if [ "$WIFI_PASS" ] ; then
        echo "psk=\"$WIFI_PASS\"" >> "$FILE"
    else
        echo "key_mgmt=NONE" >> "$FILE"
    fi
    
echo "}" >> "$FILE"
}

print_help()
{
echo "
This file configures and starts Wireless client mode and connect to AP.
Only used by HomeKit engine.

Command line arguments:
  $1 interface name
  $2 wireless ssid
  $3 password
"
}

#############################
#########  Script  ##########
#############################

#Print help
if [ $1 == "--help" ]; then
    print_help
    exit 1
fi

#Check input params
if [ $# -ne 3 ]; then
    echo "Not enough params!" >&2
    print_help    
    exit 1
fi

echo "Started script wifi_client.sh" >&2

#Broadcast the audio
if [ ! -f stationmode ]; then
    touch stationmode
    echo "touch stationmode>>>>>>>>>"
    sync
fi

#Clean and restart interface and other daemons
./wifi_disconnect.sh $INTERFACE
#rm -f /var/lib/dhcp/dhclient.leases

echo "Continue script wifi_client.sh" >&2

#Create configuration file for wpa_supplicant
create_config_file_wifi $WPA_CONF "$SSID" "$PASSWORD"

#Start wpa_supplicant in client mode
if [ -x /run/wpa_supplicant -o -x /usr/sbin/wpa_supplicant ] ; then
    wpa_supplicant -i "$INTERFACE" -D "$DRIVER" -c "$WPA_CONF" -B 2> /dev/null
    OUT=$?
    if [ $OUT -eq 127 ] ; then
        echo "Tool wpa_supplicant exited with error." >&2
        exit 1
    fi
else
    echo "Tool wpa_supplicant not found." >&2
    exit 1
fi


./sendraw $INTERFACE 5 

#Start DHCP client
wlan_hostname=$(sed s/" "/_/g $WLAN_HOSTNAME) #replace " " to "-"                            
if [ -z "$wlan_hostname" ];then
    hostname aqara_homekit
#    udhcpc -b -A 30 -i $INTERFACE -x hostname:aqara_homekit
else
    hostname $wlan_hostname
#    udhcpc -b -A 30 -i $INTERFACE -x hostname:$wlan_hostname                         
fi

#/usr/bin/ntpdate-sync silent 
#exit 0

if [ -x /sbin/dhclient ] ; then

    #if exists, kill again
    if [ -e "/var/run/dhclient.$INTERFACE.pid" ] ; then
        kill -9 $(cat /var/run/dhclient.$INTERFACE.pid) 2> /dev/null
        rm /var/run/dhclient.$INTERFACE.pid 2> /dev/null
    fi
    
    dhclient -pf /var/run/dhclient.$INTERFACE.pid "$INTERFACE" 2> /dev/null
    chmod 0777 /var/run/dhclient.$INTERFACE.pid 2> /dev/null
    OUT=$?
    if [ $OUT -eq 127 ] ; then
        echo "Tool dhclient exited with error." >&2
        exit 1
    fi
else
    echo "Tool dhclient not found." >&2
    exit 1
fi

#wait 5 seconds
#sleep 5
/usr/bin/ntpdate-sync silent &

exit 0


#Disable Power Save mode of WiFi module (for MuRata module)
echo "Setting power save mode." >&2
iw dev $INTERFACE set power_save on
sleep 2
iw dev $INTERFACE set power_save off
#sleep 2

/usr/bin/ntpdate-sync silent &

exit 0
